setfenv(1, ShinyEnv)


ShinyConfig = {}

local L = ShinyLocale



function ShinyConfig:OnInitialize()
	-- Get Option Tables
	self:CreateOptions()

	-- Register settings
	local AceConfig = LibStub("AceConfig-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	AceConfig:RegisterOptionsTable("ShinyStats", self.options, "shinystats")

	local BlizPanel = AceConfigDialog:AddToBlizOptions("ShinyStats", nil, nil, "general")
	AceConfigDialog:AddToBlizOptions("ShinyStats", "Colors", "ShinyStats", "colors")
	AceConfigDialog:AddToBlizOptions("ShinyStats", "Profiles", "ShinyStats", "profile")
end


function ShinyConfig:CreateOptions()
	self.options = {
		handler = self,
		type = 'group',
		args = {
			config = {
				name = L["ShinyStats settings"],
				desc = L["Shows the blizzard configuration dialog for ShinyStats."],
				type = 'execute',
				func = "ShowOptions",
				hidden = true,
				cmdHidden = false,
			},
			general = {
				name = "General Settings",
				type = 'group',
				args = {
					shortenGroup = {
						name = L["Enable Shortening"],
						type = "group",
						args = {
							SHORTEN_ENCHANT = {
								type = 'toggle',
								name = L["Shorten Enchant"],
								desc = L["Remove redundant words in Enchant descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							SHORTEN_SOCKET_GEM = {
								type = 'toggle',
								name = L["Shorten Socket/Gem"],
								desc = L["Remove redundant words in Socket/Gem descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							SHORTEN_EQUIP = {
								type = 'toggle',
								name = L["Shorten Equip"],
								desc = L["Remove redundant words in Equip descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							SHORTEN_USE = {
								type = 'toggle',
								name = L["Shorten Use"],
								desc = L["Remove redundant words in Use descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							SHORTEN_CHANCE = {
								type = 'toggle',
								name = L["Shorten Chance on Hit"],
								desc = L["Remove redundant words in Chance on Hit descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							SHORTEN_SEPARATOR = {
								type = 'input',
								name = L["Shorten Separator"],
								desc = L["The string replacing substituted values."],
								set = "SetOptionInput",
								get = "GetOption",
							},
						},
					},
					enableGroup = {
						name = L["Enable Parsing"],
						type = "group",
						args = {
							ENABLE_ENCHANT = {
								type = 'toggle',
								name = L["Enchant"],
								desc = L["Enables parsing in Enchant descriptions."],
								set = "SetOption",
								get = "GetOption",
							},

							ENABLE_SOCKET_GEM = {
								type = 'toggle',
								name = L["Socket/Gem"],
								desc = L["Enables parsing in Socket/Gem descriptions."],
								set = "SetOption",
								get = "GetOption",
							},
							
							ENABLE_EQUIP = {
								type = 'toggle',
								name = L["Equip"],
								desc = L["Enables parsing in Equip descriptions."],
								set = "SetOption",
								get = "GetOption",
							},
							
							ENABLE_USE = {
								type = 'toggle',
								name = L["Use"],
								desc = L["Enables parsing in Use descriptions."],
								set = "SetOption",
								get = "GetOption",
							},
							
							ENABLE_CHANCE = {
								type = 'toggle',
								name = L["Chance on Hit"],
								desc = L["Enables parsing in Chance on Hit descriptions."],
								set = "SetOption",
								get = "GetOption",
							},
							
							ENABLE_MISC = {
								type = 'toggle',
								name = L["Miscellaneous"],
								desc = L["Enable parsing of miscellaneous stats, such as Item Level, Requires, Cooldown, Class etc."],
								set = "SetOption",
								get = "GetOption",
							},

							ENABLE_PARTIAL_MATCHES = {
								type = 'toggle',
								name = L["Enable Partial Matches"],
								desc =  L["Enables matching parts of text, such as stats in Equip/Use/Chance on Hit/Enchant/Socket or Gem descriptions, instead of only entire lines."],
								set = "SetOption",
								get = "GetOption",
							},

							ENABLE_STAT_STRIP_COLORS = {
								type = 'toggle',
								name = L["Enable Stripping Embedded Colors"],
								desc =  L["Enables stripping color codes embedded in the text of stats, such as those found on Artifacts with +1 Rank: Minor Trait, where +1 Rank would become green."],
								set = "SetOption",
								get = "GetOption",
							},
						}
					},
					miscGroup = {
						name = L["Miscellaneous"],
						type = "group",
						args = {
							ENABLE_CACHE = {
								type = 'toggle',
								name = L["Enable Caching"],
								desc =  L["Enables caching of strings and their parsed result, significantly reducing CPU usage at the price of a slight increase in memory usage."],
								set = "SetOption",
								get = "GetOption",
							},

							DEBUG = {
								type = 'toggle',
								name = L["Debug"],
								desc =  L["Should normally NOT be enabled. Causes a significant performance impact."],
								set = "SetOption",
								get = "GetOption",
							},
						},
					},
				},
			},
			colors = {
				type = 'group',
				name = L["Colors"],
				desc = L["Set the colors of the various stats."],
				args = {}
			},
			profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(ShinyDB),
		},
	}

	
	for k,v in pairs(ShinyDB.profile.colors) do
		if (k ~= SHINY_PASSTHRU and COLOR_NAME_MAP[k]) then
			self.options.args.colors.args[k] = {
				name = COLOR_NAME_MAP[k],
				type = "group",
				args = {
					color = {
						order = 0,
						name = L["Pick a color"],
						type = "color",
						set = "SetColor",
						get = "GetColor",
						disabled = "DisabledColor",
					},
					enabled = {
						name =L["Enable"],
						type = "toggle",
						set = "SetToggleColor",
						get = "GetToggleColor",
					},
					spacer = {
						name = " ",
						type = "description",
						width = "full",
					},
					reset = {
						order = -1,
						name = L["Reset"],
						type = "execute",
						func = "ResetColor",
					},
				}
			}
		end
	end
end


function ShinyConfig:GetColor(info)
	local parent, node = info[#info-1], info[#info]

	return Color.hex2dec(ShinyDB.profile.colors[parent].value)
end


function ShinyConfig:SetColor(info, ...)
	local parent, node = info[#info-1], info[#info]

	local group = ShinyDB.profile.colors[parent] or {
		id = parent,
		enabled = true,
	}

	group.value = Color.dec2hex(...) -- r,g,b,a

	ShinyDB.profile.colors[parent] = group

	ShinyStats.colors[parent]:SetDECA(...)

	ShinyCache:SetTainted(true)
end


function ShinyConfig:ResetColor(info, ...)
	local parent, node = info[#info-1], info[#info]
	local defaultColor = ShinyDB.defaults.profile.colors[parent]
	local color = ShinyStats.colors[parent]

	ShinyDB.profile.colors[parent] = defaultColor

	color:SetHEXA(defaultColor.value)
	color:SetEnabled(defaultColor.enabled)

	ShinyCache:SetTainted(true)
end


function ShinyConfig:SetToggleColor(info, ...)
	local parent, node, value = info[#info-1], info[#info], ...

	local group = ShinyDB.profile.colors[parent] or {
		id = parent,
	}

	group.enabled = value

	ShinyDB.profile.colors[parent] = group
	self.options.args.colors.args[parent].args["color"].disabled = not value

	ShinyStats.colors[parent]:SetEnabled(value)

	ShinyCache:SetTainted(true)
end


function ShinyConfig:GetToggleColor(info)
	local parent, node = info[#info-1], info[#info]
	
	return ShinyDB.profile.colors[parent].enabled
end


function ShinyConfig:DisabledColor(info)
	return not self:GetToggleColor(info)
end


function ShinyConfig:GetOption(info)
	local parent, node = info[#info-1], info[#info]

	return ShinyDB.profile.options[node]
end


function ShinyConfig:SetOption(info, ...)
	local parent, node, value = info[#info-1], info[#info], ...

	ShinyDB.profile.options[node] = value

	ShinyCache:SetTainted(true)
end


function ShinyConfig:SetOptionInput(info, ...)
	local parent, node, value = info[#info-1], info[#info], ...

	ShinyDB.profile.options[node] = value:gsub([[["'|\%]+]], "")

	ShinyCache:SetTainted(true)
end


function ShinyConfig:ImportConfig()
	local version = ShinyDB.profile.version

	if (not version or version < CONFIG_VERSION) then
		local convert = {}

		if (not version or version < 2.0) then
			table.insert(convert, {
				oldRoot = ShinyDB.profile,
				newRoot = ShinyDB.profile.options,
				args = {
					{"shortenEnchants",          "SHORTEN_ENCHANT"},
					{"shortenEquips",            "SHORTEN_EQUIP"},
					{"shortenSeparator",         "SHORTEN_SEPARATOR"},
					{"enablePartialEquip",       "ENABLE_PARTIAL_MATCHES"},
					
					{"disableItemLevelColoring"},
					{"prefixEnchants"},
				}
			})

			table.insert(convert, {
				oldRoot = ShinyDB.profile.colors,
				newRoot = ShinyDB.profile.colors,
				handler = function(oldRoot, newRoot, oldKey, newKey)
							local group = newRoot[newKey] or {
								id = newKey,
								enabled = true,
								value = "ffffffff",
							}

							if (oldRoot[oldKey] ~= nil) then
								group.value = oldRoot[oldKey] -- HEX
							end

							newRoot[newKey] = group
						end,
				args = {
					{"agi",  SHINY_AGILITY},
					{"ap",   SHINY_ATTACKPOWER},
					{"ar",   SHINY_ARCANERESISTANCE},
					{"arm",  SHINY_ARMOR},
					{"blo",  SHINY_BLOCK},
					{"cri",  SHINY_CRITICAL},
					{"dmg",  SHINY_WEAPONDAMAGE},
					{"dod",  SHINY_DODGE},
					{"dps",  SHINY_WEAPONDPS},
					{"ench", SHINY_ENCHANT},
					{"exp",  SHINY_EXPERTISE},
					{"fr",   SHINY_FIRERESISTANCE},
					{"frr",  SHINY_FROSTRESISTANCE},
					{"has",  SHINY_HASTE},
					{"hit",  SHINY_HIT},
					{"ilvl", SHINY_ITEMLEVEL},
					{"int",  SHINY_INTELLECT},
					{"mr",   SHINY_MASTERY},
					{"nr",   SHINY_NATURERESISTANCE},
					{"par",  SHINY_PARRY},
					{"pvp",  SHINY_PVPPOWER},
					{"res",  SHINY_PVPRESILIENCE},
					{"sp",   SHINY_SPELLPOWER},
					{"spd",  SHINY_WEAPONSPEED},
					{"spen", SHINY_SPELLPENETRATION},
					{"spi",  SHINY_SPIRIT},
					{"sr",   SHINY_SHADOWRESISTANCE},
					{"sta",  SHINY_STAMINA},
					{"str",  SHINY_STRENGTH},
					{"vers", SHINY_VERSATILITY},
				}
			})
		end

		for _,set in ipairs(convert) do
			local oldRoot, newRoot = set.oldRoot, set.newRoot
			local handler = type(set.handler) == "function" and set.handler
			local oldMeta = getmetatable(oldRoot)

			if (oldMeta) then
				setmetatable(oldRoot, nil)
			end

			for _,map in ipairs(set.args) do
				local oldKey, newKey = unpack(map)
				if (oldRoot[oldKey] ~= nil) then
					if (handler) then
						handler(oldRoot, newRoot, oldKey, newKey)
					elseif (newKey) then
						newRoot[newKey] = oldRoot[oldKey]
					end

					oldRoot[oldKey] = nil
				end
			end

			if (oldMeta) then
				setmetatable(oldRoot, oldMeta)
			end
		end
	end
	
	ShinyDB.profile.version = CONFIG_VERSION
end


function ShinyConfig:Show()
	InterfaceOptionsFrame:Show()
	InterfaceOptionsFrame_OpenToCategory("ShinyStats")
end

